import java.awt.*;

public class PopupPictureCanvas extends PopupPanel {
    PictureCanvas P;

    public PopupPictureCanvas() {
        super("Picture Window", 1000,1000);
        P=new PictureCanvas();
        P.setSize(1000,1000);
        add(P);
        P.setBackground(Color.black);
        setVisible(true);
        F.setSize(1000,1000);
    }

    public void resized(int w, int h){
	try {P.setSize(w,h);}
	catch(Exception e) {}
    }
    
}
